@@ -8,6 +8,22 @@
  *
  *  Copyright (c) by Jaroslav Kysela <perex@jcu.cz>
  *  Based on vplay program by Michael Beck
+ *
+ *
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  */
 
 #include <stdio.h>
@@ -222,8 +238,7 @@ static void version( void )
 
 int main( int argc, char *argv[] )
 {
-  int card, dev, tmp, err;
-  char c;
+  int card, dev, tmp, err, c;
 
   card = 0;
   dev = 0;
@@ -255,7 +270,7 @@ int main( int argc, char *argv[] )
     version();
     return 0;
   }
-  while ( (c = getopt( argc, argv, "-hlc:d:qs:So:t:b:vrwuxB:c:p:mM" )) != EOF )
+  while ( (c = getopt( argc, argv, "hlc:d:qs:So:t:b:vrwuxB:c:p:mM" )) != EOF )
     switch ( c ) {
       case 'h':
         usage( command );
@@ -538,17 +553,35 @@ static void write_zeros( unsigned x )
 
 static void set_format(void)
 {
+  unsigned int bps;	/* bytes per second */
+  unsigned int size;	/* fragment size */
   struct snd_pcm_playback_params pparams;
   struct snd_pcm_record_params rparams;
 
+  bps = format.rate * format.channels;
+  switch ( format.format ) {
+    case SND_PCM_SFMT_U16_LE:
+    case SND_PCM_SFMT_U16_BE:
+      bps <<= 1;
+      break;
+    case SND_PCM_SFMT_IMA_ADPCM:
+      bps >>= 2;
+      break;
+  }
+  bps >>= 2;		/* ok.. this buffer should be 0.25 sec */
+  if ( bps < 16 ) bps = 16;
+  size = buffer_size;
+  while ( size > bps ) size >>= 1;
+  if ( size < 16 ) size = 16;
+
   if ( direction == SND_PCM_OPEN_PLAYBACK ) {
     if ( snd_pcm_playback_format( pcm_handle, &format ) < 0 ) {
       fprintf( stderr, "%s: unable to set playback format %s, %iHz, %i voices\n", command, get_format( format.format ), format.rate, format.channels );
       exit( 1 );
     }
     memset( &pparams, 0, sizeof( pparams ) );
-    pparams.fragment_size = buffer_size;
-    pparams.fragments_max = ( pinfo.buffer_size / buffer_size ) - 1;
+    pparams.fragment_size = size;
+    pparams.fragments_max = -1;		/* little trick */
     pparams.fragments_room = 1;
     if ( snd_pcm_playback_params( pcm_handle, &pparams ) < 0 ) {
       fprintf( stderr, "%s: unable to set playback params\n", command );
@@ -560,7 +593,7 @@ static void set_format(void)
       exit( 1 );
     }
     memset( &rparams, 0, sizeof( rparams ) );
-    rparams.fragment_size = buffer_size;
+    rparams.fragment_size = size;
     rparams.fragments_min = 1;
     if ( snd_pcm_record_params( pcm_handle, &rparams ) < 0 ) {
       fprintf( stderr, "%s: unable to set record params\n", command );
